import { requiredParam } from "./utils/Utils";
import { httpGET, httpPOST, httpDELETE } from "./utils/HTTPUtil";
import { platform, getPlatformController } from "./PlatformControllerFactory";

const PUSH_NOT_INITIALISED_MSG =
  "MFP Push SDK is not initialized. Please call 'initialize' method before using other SDK APIs.";
let controller;
const ACCESS_SCOPE =
  "push.mobileclient devices.read devices.write subscriptions.write";

export default class MFPPush {
  constructor() {
    controller = getPlatformController();
  }

  /**
   *
   * @param {JSON} params - JSON Object that contains the required fields to initialize the Push Service.
   * @example
   * {
   *    appId: "com.push.app",
   *    mfpServerContext: "/mfp",
   *    serverUrl: "https://mfp-server-url.com",
   *    safariWebsitePushId: "web.com.mfp-server-url",
   * }
   */
  initialize(
    params = requiredParam(
      "params",
      "Fields: ['appId', 'mfpContextRoot'] are needed in the parameter object!"
    )
  ) {
    if (params.appId && params.mfpContextRoot) {
      localStorage.setItem("appId", params.appId);
      localStorage.setItem("mfpContextRoot", params.mfpContextRoot);
    } else {
      return new Error(
        "Fields: ['appId', 'mfpContextRoot'] are mandatory in the parameter object!"
      );
    }
    if (!params.serverUrl || !params.safariWebsitePushId) {
      console.error(
        "Parameter 'serverUrl' and 'safariWebsitePushId' are not passed. Both are required for Safari Push Notifications to work."
      );
    } else {
      if (params.serverUrl) {
        let serverUrl = params.serverUrl;
        if (!serverUrl.endsWith("/")) serverUrl += "/";
        localStorage.setItem("serverUrl", params.serverUrl);
      }
      if (params.safariWebsitePushId)
        localStorage.setItem("safariWebsitePushId", params.safariWebsitePushId);
    }

    var wlInitOptions = {
      mfpContextRoot: params.mfpContextRoot,
      applicationId: params.appId,
    };
    WL.Client.init(wlInitOptions).then(function () {
      WL.AuthorizationManager.obtainAccessToken(ACCESS_SCOPE).then((token) => {
        console.log("MFP Push App initialized successfully");
      });
    });
  }

  /**
   * Registers the device to the Push Service
   * @param {JSON} params - this is an Object that contains
   * @example -
   * {
   *   "token": "{some-stringified-json-token}",
   *   "userId": "some-user-id"
   * }
   */
  async registerDevice(params) {
    if (checkPushInitialization()) {
      return controller.registerDevice(params);
    } else {
      return reject(pushNotInitialisedPromise());
    }
  }

  /**
   * Unregisters the device from the Push Service.
   */
  unregisterDevice() {
    return new Promise((resolve, reject) => {
      if (checkPushInitialization()) {
        let deviceId = WL.BrowserManager.getWLUniqueID();
        httpDELETE("/devices/" + deviceId)
          .then((res) => {
            resolve(res.responseText);
          })
          .catch((err) => {
            console.error("Failed to delete device.", err);
            reject(err);
          });
      } else {
        return reject(pushNotInitialisedPromise());
      }
    });
  }

  /**
   * Subscribes the current device to the tag given in parameter.
   * @param {string} tag - this is a String Object that contains Tag that we want to subscribe to.
   * @example -
   * "offers"
   */
  subscribe(tag) {
    return new Promise((resolve, reject) => {
      if (checkPushInitialization()) {
        var devId = WL.BrowserManager.getWLUniqueID();
        var tagsObject = {
          tagName: tag,
          deviceId: devId,
        };
        httpPOST("/subscriptions", JSON.stringify(tagsObject), {
          "Content-type": "application/json",
          Accept: "application/json",
        })
          .then((res) => {
            resolve(res);
          })
          .catch((err) => {
            console.error("Failed to subscribe to tag. ", err);
            reject(err);
          });
      } else {
        return reject(pushNotInitialisedPromise());
      }
    });
  }

  /**
   * Unsubscribes the current device from the tag given in parameter.
   * @param {string} tag - this is a String Object that contains Tag that we want to unsubscribe from.
   * @example -
   * "offers"
   */
  unsubscribe(tag) {
    return new Promise((resolve, reject) => {
      if (checkPushInitialization()) {
        var deviceId = WL.BrowserManager.getWLUniqueID();
        var queryParams = "?";
        queryParams += "deviceId=" + deviceId + "&";
        queryParams += "tagName=" + tag;
        httpDELETE("/subscriptions" + queryParams)
          .then((res) => {
            resolve(res);
          })
          .catch((err) => {
            console.error("Failed to unsubscribe from tags. ", err);
            reject(err);
          });
      } else {
        return reject(pushNotInitialisedPromise());
      }
    });
  }

  /**
   * Retrieves Active Subscription of the device.
   * @returns {Array} - this is the Array of Tags that the device is currently subscribed to.
   * @example -
   * ["premium_user"]
   */
  retrieveActiveSubscriptions() {
    return new Promise((resolve, reject) => {
      if (checkPushInitialization()) {
        var devId = WL.BrowserManager.getWLUniqueID();
        httpGET("/subscriptions?deviceId=" + devId)
          .then((res) => {
            resolve(JSON.parse(res.responseText));
          })
          .catch((err) => {
            console.error("Failed to retrieveActiveSubscriptions. ", err);
            reject(err);
          });
      } else {
        return reject(pushNotInitialisedPromise());
      }
    });
  }

  /**
   * Retrieves Subscriptions that are available to the device.
   * @returns {Array} - this is the Array of Tags that the device can subscribe to.
   * @example -
   * ["tag1", "tag2", "offers", "tag3"]
   */
  async retrieveAvailableSubscriptions() {
    return new Promise((resolve, reject) => {
      if (checkPushInitialization()) {
        httpGET("/tags")
          .then((res) => {
            resolve(JSON.parse(res.responseText));
          })
          .catch((err) => {
            console.error("Failed to retrieveAvailableSubscriptions. ", err);
            reject(err);
          });
      } else {
        return reject(pushNotInitialisedPromise());
      }
    });
  }
}

/**
 * This method returns Error when the SDK is not initialized - so as to prevent other API calls.
 */
function checkPushInitialization() {
  if (
    typeof WL == undefined ||
    !(localStorage.getItem("mfpContextRoot") && localStorage.getItem("appId"))
  ) {
    return false;
  }
  return true;
}

function pushNotInitialisedPromise() {
  return new Error(PUSH_NOT_INITIALISED_MSG);
}

export { platform };
